// SPDX-FileCopyrightText: 2019-2022 Tobias Leupold <tl at stonemx dot de>
//
// SPDX-License-Identifier: GPL-3.0-or-later

// Local includes

#include "DrawPopup.h"

#include "shared/DefaultValues.h"
#include "shared/SharedStyles.h"

// Qt includes
#include <QDebug>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QSpinBox>
#include <QRadioButton>
#include <QPushButton>
#include <QCheckBox>
#include <QApplication>
#include <QStyle>
#include <QTimer>
#include <QRandomGenerator>
#include <QMessageBox>

DrawPopup::DrawPopup(QWidget *parent, const Draw::Settings *drawSettings)
    : SelectionPopup(parent), m_drawSettings(drawSettings)
{
    m_main = new QWidget;
    QVBoxLayout *layout = new QVBoxLayout(m_main);

    m_selection = new QWidget;
    auto *selectionLayout = new QHBoxLayout(m_selection);

    m_isSinglePlayer = new QLabel(tr("<span style=\"%1\">Allein gekommener Spieler!</span>").arg(
                                     SharedStyles::redText));
    m_isSinglePlayer->setAlignment(Qt::AlignCenter);
    layout->addWidget(m_isSinglePlayer);

    selectionLayout->addWidget(new QLabel(tr("Tisch")));

    m_table = new QSpinBox;
    m_table->setMinimum(1);
    m_table->setMaximum(DefaultValues::maximumDrawTables);
    connect(m_table, QOverload<int>::of(&QSpinBox::valueChanged),
            this, &DrawPopup::checkSelection);
    selectionLayout->addWidget(m_table);

    auto *pairLayout = new QVBoxLayout;
    selectionLayout->addLayout(pairLayout);

    m_pair1 = new QRadioButton(tr("Paar 1"));
    connect(m_pair1, &QRadioButton::toggled, this, &DrawPopup::checkSelection);
    pairLayout->addWidget(m_pair1);

    m_pair2 = new QRadioButton(tr("Paar 2"));
    pairLayout->addWidget(m_pair2);

    auto *buttonLayout = new QVBoxLayout;
    selectionLayout->addLayout(buttonLayout);

    m_newDraw = new QPushButton;
    m_newDraw->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);
    m_newDraw->setIcon(QApplication::style()->standardIcon(QStyle::SP_BrowserReload));
    m_newDraw->setToolTip(tr("Neu auslosen"));
    connect(m_newDraw, &QPushButton::clicked, this, &DrawPopup::drawSeat);
    buttonLayout->addWidget(m_newDraw);

    auto *setToFirstFree = new QPushButton;
    setToFirstFree->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);
    setToFirstFree->setIcon(QApplication::style()->standardIcon(QStyle::SP_ArrowLeft));
    setToFirstFree->setToolTip(tr("Auf ersten freien Platz setzen"));
    connect(setToFirstFree, &QPushButton::clicked, this, &DrawPopup::selectFirstFreeSeat);
    buttonLayout->addWidget(setToFirstFree);

    layout->addWidget(m_selection);

    m_taken = new QLabel(tr("<span style=\"%1\">Bereits vergeben!</span>").arg(
                            SharedStyles::redText));
    m_taken->setAlignment(Qt::AlignCenter);
    layout->addWidget(m_taken);

    m_beyondLimit = new QLabel(tr("<span style=\"%1\">Tischvergabe außerhalb des Limits!"
                                  "</span>").arg(SharedStyles::redText));
    m_beyondLimit->setAlignment(Qt::AlignCenter);
    m_beyondLimit->hide();
    layout->addWidget(m_beyondLimit);

    m_save = new QPushButton(tr("Auslosung speichern"));
    connect(m_save, &QPushButton::clicked, this, &SelectionPopup::accept);
    layout->addWidget(m_save);

    m_delete = new QPushButton(tr("Auslosung löschen"));
    connect(m_delete, &QPushButton::clicked, this, &DrawPopup::confirmDelete);
    layout->addWidget(m_delete);

    setScrollWidget(m_main);
}

void DrawPopup::show(Draw::Seat seat, bool empty, bool allRoundsFinished, bool isSinglePlayer,
                     const QPoint &position)
{
    m_allRoundsFinished = allRoundsFinished;

    if (m_drawSettings->consecutive) {
        // We want to do a consecutive assignment
        m_drawingPossible = false;

    } else {
        // We want to do a random draw

        // Check if we can draw at all and get all seats we can draw

        int maximumTable = 0;

        if (m_drawSettings->consecutiveDraw != -1) {
            // We do have a limit for drawing

            if (m_firstFreeSeat.table >= m_drawSettings->consecutiveDraw) {
                // We're already above it --> no drawing anymore
                m_drawingPossible = false;

            } else {
                // We can draw

                if (m_drawSettings->randomDraw != -1
                    && m_firstFreeSeat.table <= m_drawSettings->randomDraw) {
                    // There's a limit up to which we can freely draw and we are below it

                    m_drawingPossible = true;
                    maximumTable = m_drawSettings->randomDraw;

                } else {
                    // Either, there's no range for drawing freely or we're above it, so we do
                    // a close table drawing up to the consecutive drawing limit
                    m_drawingPossible = true;
                    maximumTable = qMin(
                        m_firstFreeSeat.table + m_drawSettings->windowDraw,
                        m_drawSettings->consecutiveDraw - 1);
                }
            }

        } else {
            // There's no limit for drawing

            if (m_drawSettings->randomDraw != -1
                && m_firstFreeSeat.table <= m_drawSettings->randomDraw) {
                // There's a limit up to which we can freely draw and we are below it

                m_drawingPossible = true;
                maximumTable = m_drawSettings->randomDraw;

            } else {
                // Either, there's no range for drawing freely or we're above it, so we do
                // a close table drawing up to the consecutive drawing limit
                m_drawingPossible = true;
                maximumTable = m_firstFreeSeat.table + m_drawSettings->windowDraw;
            }
        }

        if (m_drawSettings->tablesLimit != -1 && maximumTable > m_drawSettings->tablesLimit) {
            maximumTable = m_drawSettings->tablesLimit;
        }

        if (m_drawingPossible) {
            m_drawableSeats.clear();
            for (int table = m_firstFreeSeat.table; table <= maximumTable; table++) {
                for (int pair = 1; pair < 3; pair++) {
                    const auto seat = Draw::Seat { table, pair };
                    if (! isSeatTaken(seat)) {
                        m_drawableSeats.append(seat);
                    }
                }
            }
        }

        if (m_drawableSeats.isEmpty()) {
            m_drawingPossible = false;
        }
    }

    m_isSinglePlayer->setVisible(isSinglePlayer);
    m_newDraw->setEnabled(m_drawingPossible);

    selectSeat(seat.isEmpty() ? (m_drawingPossible ? getRandomSeat() : m_firstFreeSeat)
                              : seat);

    m_beyondLimit->setVisible(m_drawSettings->tablesLimit != -1
                              && m_table->value() > m_drawSettings->tablesLimit);

    m_selection->setEnabled(empty);
    m_taken->hide();
    m_save->setVisible(empty);
    m_delete->setVisible(! empty);

    if (empty) {
        checkSelection();
    }

    m_main->adjustSize();

    if (position == QPoint(-1, -1)) {
        SelectionPopup::show();
    } else {
        SelectionPopup::show(position);
    }
}

void DrawPopup::confirmDelete()
{
    if (! m_allRoundsFinished) {
        // We can't use the static convenience functions here, because the message box's parent
        // has to be the popup's parent, so that it doesn't inherit the popup's style sheet and
        // will be displayed centered on the window.
        QMessageBox messageBox(qobject_cast<QWidget *>(parent()));
        messageBox.setIcon(QMessageBox::Warning);
        messageBox.setWindowTitle(tr("Auslosung löschen"));
        messageBox.setText(tr(
            "<p><b>Auslosung löschen</b></p>"
            "<p>Es wurden in mindestens einer Runde noch nicht alle Ergebnisse eingegeben. Wenn "
            "die Auslosung geändert wird, dann wird das evtl. dazu führen, dass manche Ergebnisse "
            "nicht entsprechend der Auslosung eingegeben werden können, weil bereits ein Ergebnis "
            "für einen anders ausgelosten Tisch eingegeben wurde.</p>"
            "<p>Soll die Auslosung wirklich gelöscht werden?"));
        messageBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
        messageBox.setDefaultButton(QMessageBox::No);

        if (messageBox.exec() == QMessageBox::No) {
            reject();
            return;
        }
    }

    SelectionPopup::accept();
}

Draw::Seat DrawPopup::getRandomSeat() const
{
    return m_drawableSeats.at(QRandomGenerator::global()->bounded(m_drawableSeats.count()));
}

void DrawPopup::setAssignedTables(QVector<int> &assignedPair1Tables,
                                        QVector<int> &assignedPair2Tables)
{
    m_assignedPair1Tables = assignedPair1Tables;
    m_assignedPair2Tables = assignedPair2Tables;

    const int firstFreePair1Table = findFirstGap(m_assignedPair1Tables);
    const int firstFreePair2Table = findFirstGap(m_assignedPair2Tables);
    m_firstFreeSeat = firstFreePair1Table <= firstFreePair2Table
                          ? Draw::Seat { firstFreePair1Table, 1 }
                          : Draw::Seat { firstFreePair2Table, 2 };
}

int DrawPopup::findFirstGap(const QVector<int> &list) const
{
    int firstGap = 1;

    if (m_assignmentsPerPair == 1) {
        // It's a bit easier for fixed pairs tournaments ...
        for (const int i : list) {
            if (i != firstGap) {
                return firstGap;
            }
            firstGap++;
        }
    } else {
        // ... and a bit more complicated for single players ;-)
#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
        QVectorIterator<int> table(list);
#else
        QListIterator<int> table(list);
#endif
        while (table.hasNext()) {
            table.next();
            if (table.hasNext() && table.next() == firstGap) {
                firstGap++;
            } else {
                return firstGap;
            }
        }
    }

    return firstGap;
}

void DrawPopup::selectSeat(const Draw::Seat &seat)
{
    m_table->blockSignals(true);
    m_pair1->blockSignals(true);
    m_pair2->blockSignals(true);
    m_table->setValue(seat.table);
    m_pair1->setChecked(seat.pair == 1);
    m_pair2->setChecked(seat.pair == 2);
    m_table->blockSignals(false);
    m_pair1->blockSignals(false);
    m_pair2->blockSignals(false);
}

void DrawPopup::selectFirstFreeSeat()
{
    selectSeat(m_firstFreeSeat);
    checkSelection();
}

void DrawPopup::drawSeat()
{
    selectSeat(getRandomSeat());
    checkSelection();
}

bool DrawPopup::isSeatTaken(const Draw::Seat &seat) const
{
    return seat.pair == 1 ? m_assignedPair1Tables.count(seat.table) == m_assignmentsPerPair
                          : m_assignedPair2Tables.count(seat.table) == m_assignmentsPerPair;
}

void DrawPopup::checkSelection()
{
    const bool taken = isSeatTaken(Draw::Seat { m_table->value(), m_pair1->isChecked() ? 1 : 2 });
    m_taken->setVisible(taken);
    m_save->setVisible(! taken);
}

Draw::Seat DrawPopup::seat() const
{
    return { m_selection->isEnabled() ? m_table->value() : 0,
             m_pair1->isChecked() ? 1 : 2 };
}

void DrawPopup::setPairMode(const bool state)
{
    m_assignmentsPerPair = state ? 1 : 2;
}

QVector<Draw::Seat> DrawPopup::completeDraw(int seats) const
{
    QVector<Draw::Seat> allSeats;
    for (int table = 1; table <= seats / 2 / m_assignmentsPerPair; table++) {
        for (int pair = 1; pair <= 2; pair++) {
            for (int i = 0; i < m_assignmentsPerPair; i++) {
                allSeats.append(Draw::Seat { table, pair });
            }
        }
    }

    QVector<Draw::Seat> assignment;
    QVector<int> index;
    for (int i = 0; i < seats; i++) {
        index.append(i);
    }

    while (index.count() > 0) {
        const int i = QRandomGenerator::global()->bounded(index.count());
        assignment.append(allSeats.at(index.at(i)));
        index.remove(i);
    }

    return assignment;
}
