// SPDX-FileCopyrightText: 2010-2020 Tobias Leupold <tl at stonemx dot de>
//
// SPDX-License-Identifier: GPL-3.0-or-later

// Local includes
#include "TableDelegate.h"

// Qt includes
#include <QPainter>
#include <QBrush>
#include <QDebug>
#include <QGuiApplication>
#include <QPalette>

TableDelegate::TableDelegate(QObject *parent, HighlightMode highlightMode)
    : QStyledItemDelegate(parent), m_highlightMode(highlightMode)
{
    const bool darkMode = qApp->property("DARK_MODE").toBool();
    const QPalette palette;

    if (! darkMode) {
        m_topLineColor = palette.color(QPalette::Mid);
    } else {
        m_topLineColor = palette.color(QPalette::Light);
    }

    m_alternateBackground = QPixmap(1, 1);
#ifndef Q_OS_MACOS
    m_alternateBackground.fill(palette.color(QPalette::AlternateBase));
#else
    // If we're on macOS and in dark mode, QPalette::AlternateBase doesn't look nice.
    // We use a color derived from the default base in this case:
    if (! darkMode) {
        m_alternateBackground.fill(palette.color(QPalette::AlternateBase));
    } else {
        m_alternateBackground.fill(palette.color(QPalette::Base).lighter(140));
    }
#endif
}

void TableDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option,
                          const QModelIndex &index) const
{
    painter->save();

    bool drawBackground = false;
    bool drawTopLine = false;

    if ((m_highlightMode == HighlightMode::Alternating && index.row() % 2 != 0)
        || (m_highlightMode == HighlightMode::NRows && index.row() / m_highlightRows % 2 != 0)) {

        drawBackground = true;
    }

    if (m_highlightMode == HighlightMode::Custom) {
        drawBackground = index.data(Display::DrawBackground).isValid();
    }

    if (drawBackground) {
        QBrush brush = option.backgroundBrush;
        brush.setTexture(m_alternateBackground);
        painter->fillRect(option.rect, brush);
    }

    if (m_highlightMode != HighlightMode::Custom
        && index.row() != 0
        && (m_highlightMode == HighlightMode::Alternating || index.row() % m_highlightRows == 0)) {

        drawTopLine = true;
    }

    if (m_highlightMode == HighlightMode::Custom) {
        drawTopLine = index.data(Display::DrawTopLine).isValid();
    }

    if (drawTopLine) {
        painter->setPen(m_topLineColor);
        painter->drawLine(option.rect.topLeft(), option.rect.topRight());
    }

    painter->restore();

    drawText(painter, option, index);
}

void TableDelegate::drawText(QPainter *painter, const QStyleOptionViewItem &option,
                             const QModelIndex &index) const
{
    QStyledItemDelegate::paint(painter, option, index);
}

void TableDelegate::setHighlighRows(int rows)
{
    m_highlightRows = rows;
}
